"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/**
 * Create a mock function that remembers calls.
 * @param implementation function to forward calls to.
 */
function mockFn(implementation) {
    if (implementation === void 0) { implementation = function () { }; }
    if (implementation !== undefined && typeof implementation !== 'function') {
        throw new TypeError('argument 0 of mockFn must be a function');
    }
    var mock = function mockFunction() {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        mock.calls.push(args);
        if (mock.pendingImplementations.length > 0) {
            var implementation_1 = mock.pendingImplementations.shift();
            return implementation_1.apply(null, args);
        }
        return mock.implementation.apply(null, args);
    };
    mock.implementation = implementation;
    mock.pendingImplementations = [];
    mock.calls = [];
    mock.isMockFunction = true;
    mock.returns = function (value) {
        mock.implementation = function () { return value; };
        return mock;
    };
    mock.returnsOnce = function (value) {
        mock.pendingImplementations.push(function () { return value; });
        return mock;
    };
    mock.throws = function (error) {
        if (error === void 0) { error = new Error(); }
        mock.implementation = function () { throw error; };
        return mock;
    };
    mock.throwsOnce = function (error) {
        if (error === void 0) { error = new Error(); }
        mock.pendingImplementations.push(function () { throw error; });
        return mock;
    };
    mock.resolves = function (value) {
        mock.implementation = function () { return Promise.resolve(value); };
        return mock;
    };
    mock.resolvesOnce = function (value) {
        mock.pendingImplementations.push(function () { return Promise.resolve(value); });
        return mock;
    };
    mock.rejects = function (error) {
        if (error === void 0) { error = new Error(); }
        mock.implementation = function () { return Promise.reject(error); };
        return mock;
    };
    mock.rejectsOnce = function (error) {
        if (error === void 0) { error = new Error(); }
        mock.pendingImplementations.push(function () { return Promise.reject(error); });
        return mock;
    };
    return mock;
}
exports.mockFn = mockFn;
function isMockFunction(value) {
    return typeof value === 'function' && value.isMockFunction;
}
exports.isMockFunction = isMockFunction;
