"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils");
var mock_1 = require("../mock");
function validateCalledWith(expected) {
    return function (actual) {
        var expectedString = utils_1.stringify(expected);
        var actualString = utils_1.stringify(actual);
        if (!mock_1.isMockFunction(actual)) {
            return {
                success: false,
                message: "expected " + actualString + " to be called with " + expectedString + ", " +
                    "but it is not a mock function",
                negatedMessage: ''
            };
        }
        if (actual.calls.length === 0) {
            return {
                success: false,
                message: "expected " + actualString + " to be called with " + expectedString + ", " +
                    "but it was never called",
                negatedMessage: ''
            };
        }
        var args = actual.calls[actual.calls.length - 1];
        var differences = utils_1.deepCompare(args, expected);
        return {
            success: differences.length === 0,
            message: "expected " + actualString + " to be called with " + expectedString + ", " +
                ("but it was called with " + utils_1.stringify(args) + "\n") +
                utils_1.formatDifferences(differences),
            negatedMessage: "expected " + actualString + " not to be called with " +
                (expectedString + ", but it was")
        };
    };
}
exports.validateCalledWith = validateCalledWith;
