"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("../utils");
function validateThrows(constructor, message) {
    if (constructor !== undefined && typeof constructor !== 'function') {
        throw new TypeError('argument 0 of .toThrow must be an Error instance or an Error constructor');
    }
    if (message !== undefined && (typeof message !== 'string' &&
        !(message instanceof RegExp))) {
        throw new TypeError('argument 1 of .toThrow must be a string or regular expression');
    }
    if (constructor === undefined && message !== undefined) {
        throw new TypeError('argument 0 of .toThrow must not be undefined if argument 1 is present');
    }
    return function (actual) {
        var actualString = utils_1.stringify(actual);
        var throwMessage = createThrowMessage(constructor, message);
        if (typeof actual !== 'function') {
            return {
                success: false,
                message: "expected " + actualString + " to " + throwMessage + ", " +
                    'but it is not a function',
                negatedMessage: ''
            };
        }
        try {
            actual();
            return {
                success: false,
                message: "expected " + actualString + " to " + throwMessage + ", " +
                    'but it did not throw anything',
                negatedMessage: ''
            };
        }
        catch (e) {
            return {
                success: validateThrowResult(e, constructor, message),
                message: "expected " + actualString + " to " + throwMessage + ", " +
                    ("but instead it threw " + utils_1.stringify(e)),
                negatedMessage: "expected " + actualString + " not to " + throwMessage + ", but it did"
            };
        }
    };
}
exports.validateThrows = validateThrows;
function createThrowMessage(constructor, message) {
    var throwMessage = 'throw';
    if (constructor !== undefined) {
        throwMessage += " " + constructor.name;
    }
    if (typeof message === 'string') {
        throwMessage += " with a message of " + JSON.stringify(message);
    }
    if (message instanceof RegExp) {
        throwMessage += " with a message matching " + message;
    }
    return throwMessage;
}
function validateThrowResult(e, constructor, message) {
    if (constructor !== undefined) {
        if (e instanceof constructor) {
            if (typeof message === 'string') {
                return !!e && e.message === message;
            }
            if (message instanceof RegExp) {
                return !!e && message.test(e.message);
            }
            return true;
        }
        return false;
    }
    return true;
}
